package xyz.lychee.lagfixer.modules;

import com.google.common.collect.ImmutableList;
import org.bukkit.entity.Boat;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Minecart;
import org.bukkit.event.EventHandler;
import org.bukkit.event.HandlerList;
import org.bukkit.event.Listener;
import org.bukkit.event.vehicle.VehicleCreateEvent;
import xyz.lychee.lagfixer.LagFixer;
import xyz.lychee.lagfixer.managers.ModuleManager;
import xyz.lychee.lagfixer.objects.AbstractModule;
import xyz.lychee.lagfixer.objects.AbstractModule.Impact;
import xyz.lychee.lagfixer.utils.ReflectionUtils;

public class CustomVehiclesModule extends AbstractModule implements Listener {
   private NMS customVehicles;
   private boolean minecart;
   private boolean boat;

   public CustomVehiclesModule(LagFixer plugin, ModuleManager manager) {
      super(plugin, manager, Impact.LOW, "CustomVehicles", ImmutableList.of("Optimizes all vehicles such as Boats and Minecarts.", "Removes minecarts with chests spawned in mineshafts.", "Particularly useful when minecarts are frequently used on the server.", "Enhances server performance by optimizing vehicle mechanics and removing unnecessary entities."), "eyJ0ZXh0dXJlcyI6eyJTS0lOIjp7InVybCI6Imh0dHA6Ly90ZXh0dXJlcy5taW5lY3JhZnQubmV0L3RleHR1cmUvYTJjMjA1MGVjYTBlZmRkMDMxZTY1OGI5OTZjMjM5YmY3ZGEzYWVmODY1NjEyMzY3ZWQ5ZDg5NWFlN2EwZGE5ZiJ9fX0=");
   }

   @EventHandler
   public void onSpawn(VehicleCreateEvent e) {
      if (!e.isCancelled() && this.canContinue(e.getVehicle().getWorld())) {
         this.customVehicles.setCustomVehicle(e.getVehicle());
      }
   }

   public boolean isEnabled(Entity ent) {
      return ent instanceof Minecart && this.minecart || ent instanceof Boat && this.boat;
   }

   public void load() {
      this.getPlugin().getServer().getPluginManager().registerEvents(this, this.getPlugin());
   }

   public boolean loadConfig() {
      this.customVehicles = (NMS)ReflectionUtils.createInstance("CustomVehicles", this);
      this.minecart = this.getSection().getBoolean("minecart.enabled");
      this.boat = this.getSection().getBoolean("boat.enabled");
      return this.customVehicles != null;
   }

   public void disable() {
      HandlerList.unregisterAll(this);
   }

   public NMS getCustomVehicles() {
      return this.customVehicles;
   }

   public boolean isMinecart() {
      return this.minecart;
   }

   public boolean isBoat() {
      return this.boat;
   }

   public abstract static class NMS {
      private final CustomVehiclesModule module;

      public NMS(CustomVehiclesModule module) {
         this.module = module;
      }

      public abstract boolean isDefaultVehicle(Entity var1);

      public abstract void setCustomVehicle(Entity var1);

      public CustomVehiclesModule getModule() {
         return this.module;
      }
   }
}
